@@ -6,25 +6,33 @@
 #  See http://subversion.tigris.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+#    Licensed to the Subversion Corporation (SVN Corp.) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The SVN Corp. licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 import re, sys
 
-import main, tree, wc  # general svntest routines in this module.
-from svntest import Failure
+import svntest
+
 
 ######################################################################
 # Exception types
 
-class SVNUnexpectedOutput(Failure):
+class SVNUnexpectedOutput(svntest.Failure):
   """Exception raised if an invocation of svn results in unexpected
   output of any kind."""
   pass
@@ -63,13 +71,13 @@ class SVNIncorrectDatatype(SVNUnexpectedOutput):
 ######################################################################
 # Comparison of expected vs. actual output
 
-def createExpectedOutput(expected, match_all=True):
+def createExpectedOutput(expected, output_type, match_all=True):
   """Return EXPECTED, promoted to an ExpectedOutput instance if not
   None.  Raise SVNIncorrectDatatype if the data type of EXPECTED is
   not handled."""
-  if isinstance(expected, type([])):
+  if isinstance(expected, list):
     expected = ExpectedOutput(expected)
-  elif isinstance(expected, type('')):
+  elif isinstance(expected, str):
     expected = RegexOutput(expected, match_all)
   elif expected == AnyOutput:
     expected = AnyOutput()
@@ -78,11 +86,14 @@ def createExpectedOutput(expected, match_all=True):
   return expected
 
 class ExpectedOutput:
-  """Contains expected output, and performs comparisions."""
+  """Contains expected output, and performs comparisons."""
   def __init__(self, output, match_all=True):
-    """Set SELF.output (which may be None).  If MATCH_ALL is True,
-    require that all lines from OUTPUT match when performing
-    comparsisons.  If False, allow any lines to match."""
+    """Initialize the expected output to OUTPUT which is a string, or a list
+    of strings, or None meaning an empty list. If MATCH_ALL is True, the
+    expected strings will be matched with the actual strings, one-to-one, in
+    the same order. If False, they will be matched with a subset of the
+    actual strings, one-to-one, in the same order, ignoring any other actual
+    strings among the matching ones."""
     self.output = output
     self.match_all = match_all
     self.is_reg_exp = False
@@ -104,10 +115,10 @@ class ExpectedOutput:
       actual = other
 
     if isinstance(actual, list):
-      if isinstance(expected, type('')):
+      if isinstance(expected, str):
         expected = [expected]
       is_match = self.is_equivalent_list(expected, actual)
-    elif isinstance(actual, type('')):
+    elif isinstance(actual, str):
       is_match = self.is_equivalent_line(expected, actual)
     else: # unhandled type
       is_match = False
@@ -120,12 +131,26 @@ class ExpectedOutput:
   def is_equivalent_list(self, expected, actual):
     "Return whether EXPECTED and ACTUAL are equivalent."
     if not self.is_reg_exp:
-      if len(expected) != len(actual):
-        return False
-      for i in range(0, len(actual)):
-        if not self.is_equivalent_line(expected[i], actual[i]):
+      if self.match_all:
+        # The EXPECTED lines must match the ACTUAL lines, one-to-one, in
+        # the same order.
+        if len(expected) != len(actual):
           return False
-      return True
+        for i in range(0, len(actual)):
+          if not self.is_equivalent_line(expected[i], actual[i]):
+            return False
+        return True
+      else:
+        # The EXPECTED lines must match a subset of the ACTUAL lines,
+        # one-to-one, in the same order, with zero or more other ACTUAL
+        # lines interspersed among the matching ACTUAL lines.
+        i_expected = 0
+        for actual_line in actual:
+          if self.is_equivalent_line(expected[i_expected], actual_line):
+            i_expected += 1
+            if i_expected == len(expected):
+              return True
+        return False
     else:
       expected_re = expected[0]
       # If we want to check that every line matches the regexp
@@ -138,7 +163,7 @@ class ExpectedOutput:
         all_lines_match_re = False
 
       # If a regex was provided assume that we actually require
-      # some output. Fail if we don't.
+      # some output. Fail if we don't have any.
       if len(actual) == 0:
         return False
 
@@ -182,7 +207,7 @@ class AnyOutput(ExpectedOutput):
 
   def display_differences(self, message, label, actual):
     if message:
-      print message
+      print(message)
 
 class RegexOutput(ExpectedOutput):
   def __init__(self, output, match_all=True, is_reg_exp=True):
@@ -198,7 +223,7 @@ class RegexOutput(ExpectedOutput):
     display_lines(message, label, self.output, actual, True, False)
 
 class UnorderedOutput(ExpectedOutput):
-  """Marks unordered output, and performs comparisions."""
+  """Marks unordered output, and performs comparisons."""
 
   def __cmp__(self, other):
     "Handle ValueError."
@@ -255,13 +280,13 @@ class UnorderedRegexOutput(UnorderedOutput, RegexOutput):
 def display_trees(message, label, expected, actual):
   'Print two trees, expected and actual.'
   if message is not None:
-    print message
+    print(message)
   if expected is not None:
-    print 'EXPECTED', label + ':'
-    tree.dump_tree(expected)
+    print('EXPECTED %s:' % label)
+    svntest.tree.dump_tree(expected)
   if actual is not None:
-    print 'ACTUAL', label + ':'
-    tree.dump_tree(actual)
+    print('ACTUAL %s:' % label)
+    svntest.tree.dump_tree(actual)
 
 
 def display_lines(message, label, expected, actual, expected_is_regexp=None,
@@ -270,7 +295,7 @@ def display_lines(message, label, expected, actual, expected_is_regexp=None,
   with LABEL) followed by ACTUAL (also labeled with LABEL).
   Both EXPECTED and ACTUAL may be strings or lists of strings."""
   if message is not None:
-    print message
+    print(message)
   if expected is not None:
     output = 'EXPECTED %s' % label
     if expected_is_regexp:
@@ -278,26 +303,34 @@ def display_lines(message, label, expected, actual, expected_is_regexp=None,
     if expected_is_unordered:
       output += ' (unordered)'
     output += ':'
-    print output
-    map(sys.stdout.write, expected)
+    print(output)
+    for x in expected:
+      sys.stdout.write(x)
     if expected_is_regexp:
-      map(sys.stdout.write, '\n')
+      sys.stdout.write('\n')
   if actual is not None:
-    print 'ACTUAL %s:' % label
-    map(sys.stdout.write, actual)
+    print('ACTUAL %s:' % label)
+    for x in actual:
+      sys.stdout.write(x)
 
 def compare_and_display_lines(message, label, expected, actual,
-                              raisable=main.SVNLineUnequal):
+                              raisable=None):
   """Compare two sets of output lines, and print them if they differ,
   preceded by MESSAGE iff not None.  EXPECTED may be an instance of
   ExpectedOutput (and if not, it is wrapped as such).  RAISABLE is an
   exception class, an instance of which is thrown if ACTUAL doesn't
   match EXPECTED."""
+  if raisable is None:
+    raisable = svntest.main.SVNLineUnequal
   ### It'd be nicer to use createExpectedOutput() here, but its
   ### semantics don't match all current consumers of this function.
   if not isinstance(expected, ExpectedOutput):
     expected = ExpectedOutput(expected)
 
+  if isinstance(actual, str):
+    actual = [actual]
+  actual = [line for line in actual if not line.startswith('DBG:')]
+
   if expected != actual:
     expected.display_differences(message, label, actual)
     raise raisable
@@ -313,8 +346,8 @@ def verify_outputs(message, actual_stdout, actual_stderr,
   ACTUAL_STDOUT to match, every line in ACTUAL_STDOUT must match the
   EXPECTED_STDOUT regex, unless ALL_STDOUT is false.  For
   EXPECTED_STDERR regexes only one line in ACTUAL_STDERR need match."""
-  expected_stderr = createExpectedOutput(expected_stderr, False)
-  expected_stdout = createExpectedOutput(expected_stdout, all_stdout)
+  expected_stderr = createExpectedOutput(expected_stderr, 'stderr', False)
+  expected_stdout = createExpectedOutput(expected_stdout, 'stdout', all_stdout)
 
   for (actual, expected, label, raisable) in (
       (actual_stderr, expected_stderr, 'STDERR', SVNExpectedStderr),
@@ -322,11 +355,10 @@ def verify_outputs(message, actual_stdout, actual_stderr,
     if expected is None:
       continue
 
-    expected = createExpectedOutput(expected)
     if isinstance(expected, RegexOutput):
-      raisable = main.SVNUnmatchedError
+      raisable = svntest.main.SVNUnmatchedError
     elif not isinstance(expected, AnyOutput):
-      raisable = main.SVNLineUnequal
+      raisable = svntest.main.SVNLineUnequal
 
     compare_and_display_lines(message, label, expected, actual, raisable)
 
@@ -336,9 +368,7 @@ def verify_exit_code(message, actual, expected,
   if they don't match, print the difference (preceded by MESSAGE iff
   not None) and raise an exception."""
 
-  # main.spawn_process() (by virtue of main.wait_on_pipe()) will return an
-  # exit code of None on platforms not supporting Popen3
-  if actual is not None and expected != actual:
+  if expected != actual:
     display_lines(message, "Exit Code",
                   str(expected) + '\n', str(actual) + '\n')
     raise raisable
